#ifndef CLICK_NEXTHOP_HH
#define CLICK_NEXTHOP_HH
#include <click/element.hh>
#include <click/ipaddress.hh>
CLICK_DECLS

/*
=c 
NextHop

=s Forwarding Network
increases next-hop index in Forwarding Header and sets destination
annotation to the next hop

=io
1 input, 2 outputs

=processing
Push

=d

If the next-hop index is not pointing at the last hop, the index will
be increased by one and the destination annotation set to the next
hop. The packet will be send to output 0. If the next-hop index is
pointing at the last hop, the packet will be send unmodified to output 1.
Input which does not have a valid Forwarding Header is discarded.

=a RevRoute, ForwHeader, ForwHeaderPrint */

class NextHop : public Element {
public:
  NextHop();
  ~NextHop();
  const char *class_name() const { return "NextHop"; }
  const char *port_count() const { return "1/2"; }
  const char *processing() const { return PUSH; }
  void push(int port, Packet *p);
};

CLICK_ENDDECLS
#endif
