// run configure with --enable-local
// run make elemlist

// 
// IPFlowID flow(p); in iprewriter.cc instantiates a IPFlowID
// can also be instantiated from click_ip*

// before that make sure that IP packet is long enough so that this
// won't give a invalid memory access, see
// CheckIPHeader::simple_action(Packet *p)
// IPFlowID flow(reinterpret_cast<click_ip*>(p->data() + 4 + 4*p->data()[0]))

#include <click/config.h>
#include "RevRoute.hh"
#include <string.h>
#include <click/straccum.hh>

CLICK_DECLS

RevRoute::ForwardingHeader::ForwardingHeader() : len(0), data(NULL)
{
}

RevRoute::ForwardingHeader::ForwardingHeader(const unsigned char* d,bool rev)
{
  len=4+4*(int)d[0];
  data=new unsigned char[len];
  if(!rev)
    memcpy(data,d,len);
  else
    {
      data[0]=d[0]; // copy length
      data[1]=0; // set index to 0
      data[2]=d[2] ^ 0x01; // flip connection method
      data[3]=d[3];
      for(int i=0;i<d[0];i++)
	*((int*)(data+4+4*i))=*((int*)(d+len-4-4*i));
    }
}

RevRoute::ForwardingHeader::ForwardingHeader(const ForwardingHeader& f) : len(f.len)
{
  if(f.data)
    {
      data=new unsigned char[len];
      memcpy(data,f.data,len);
    }
  else
    data=NULL;
}

RevRoute::ForwardingHeader&
RevRoute::ForwardingHeader::operator=(const ForwardingHeader& f)
{
  if(data)
    delete data;
  len=f.len;
  if(f.data)
    {
      data=new unsigned char[len];
      memcpy(data,f.data,len);
    }
  else
    data=NULL;
  return *this;
}

RevRoute::ForwardingHeader::~ForwardingHeader()
{
  if(data)
    delete []data;
}

WritablePacket* RevRoute::ForwardingHeader::apply(Packet *p) const
{
  if(WritablePacket *q=p->push(len))
    {
      if(data)
	memcpy(q->data(),data,len);
      return q;
    }

  return NULL;
}

RevRoute::RevRoute() 
{
}

RevRoute::~RevRoute()
{
}

void RevRoute::push(int port, Packet *p)
{
  if(port==0)
    {
      ForwardingHeader fh(p->data(),true);
      p->pull(4+4*(int)(p->data()[0])); // strip forwarding header
      p->set_network_header(p->data(),
                            (reinterpret_cast<const click_ip*>(p->data()))->ip_hl << 2);
      IPFlowID flow(p);
      int ip_p=p->ip_header()->ip_p;
      if(ip_p != IP_PROTO_TCP && ip_p!= IP_PROTO_UDP)
	return;
      Map& map=(ip_p == IP_PROTO_TCP ? _tcp_map : _udp_map);
      if(0) {
          StringAccum sa;
          sa << "inside RevRoute setting " << flow << " -> ";
          if(fh.data)
          {
              sa << "Forw options=" << (unsigned int)(fh.data[3]);
              sa << ", method=" << (unsigned int)(fh.data[2]) << ", hops=";
              for(unsigned int i=0;i<(unsigned int)(fh.data[0]);i++)
              {
                  if(i == (unsigned int)(fh.data[1]))
                      sa << ">";
                  sa << IPAddress(fh.data+4+4*i) << "  ";
              }
          }
          click_chatter("%s",sa.c_str());      
      }
      map.set(flow.reverse(),fh);
      output(0).push(p);
    }
  else
    {
      IPFlowID flow(p);
      int ip_p=p->ip_header()->ip_p;
      if(ip_p != IP_PROTO_TCP && ip_p!= IP_PROTO_UDP)
	return;
      Map& map=(ip_p == IP_PROTO_TCP ? _tcp_map : _udp_map);
      const ForwardingHeader& fh=map.get(flow);
      // Stringaccum << flow works to print out flow ID
      StringAccum sa;
      sa << "inside RevRoute" << flow << " -> ";
      if(fh.data)
	{
	  sa << "Forw options=" << (unsigned int)(fh.data[3]);
	  sa << ", method=" << (unsigned int)(fh.data[2]) << ", hops=";
	  for(unsigned int i=0;i<(unsigned int)(fh.data[0]);i++)
	    {
	      if(i == (unsigned int)(fh.data[1]))
		sa << ">";
	      sa << IPAddress(fh.data+4+4*i) << "  ";
	    }
	}
      output(1).push(fh.apply(p));
      sa << " / ";
      if(fh.data)
	{
	  sa << "Forw options=" << (unsigned int)(fh.data[3]);
	  sa << ", method=" << (unsigned int)(fh.data[2]) << ", hops=";
	  for(unsigned int i=0;i<(unsigned int)(fh.data[0]);i++)
	    {
	      if(i == (unsigned int)(fh.data[1]))
		sa << ">";
	      sa << IPAddress(fh.data+4+4*i) << "  ";
	    }
	}
      ForwardingHeader fh2=map.get(flow);
      sa << " / ";
      if(fh2.data)
	{
	  sa << "Forw options=" << (unsigned int)(fh2.data[3]);
	  sa << ", method=" << (unsigned int)(fh2.data[2]) << ", hops=";
	  for(unsigned int i=0;i<(unsigned int)(fh2.data[0]);i++)
	    {
	      if(i == (unsigned int)(fh2.data[1]))
		sa << ">";
	      sa << IPAddress(fh2.data+4+4*i) << "  ";
	    }
	}
      click_chatter("%s",sa.c_str());
    }
}

CLICK_ENDDECLS
EXPORT_ELEMENT(RevRoute)
