#ifndef CLICK_REVROUTE_HH
#define CLICK_REVROUTE_HH
#include <click/element.hh>
#include <click/ipflowid.hh>
#include <click/hashtable.hh>
#include <clicknet/ip.h>
CLICK_DECLS

/*
=c 
RevRoute

=s Forwarding Network
saves Forwarding Headers for flows, applies the Forwarding Header with
reversed route to each reverse flow

=io
2 inputs, 2 outputs

=processing
Push

=d

Expects ForwardingHeaders on port 0 and IP packets on port 1. For each
packet on port 0, the Forwarding Header is stripped and the packet
emitted on port 0 and if the
encapsulated payload was an IP packet a mapping
(DADDR,DPORT,SADDR,SPORT,PROTO) =>
ForwHeader(ReverseRoute,ReverseMethod,Options)
is installed (ReverseMethod has the 0-bit inverted).
IP Packets on port 1 are encapsulated in Forwarding Headers according
to the above mapping.

=a NextHop, ForwHeader, ForwHeaderPrint, IPRewriter
*/

class RevRoute : public Element {
public:
  class ForwardingHeader {
  public:
    ForwardingHeader();
    ForwardingHeader(const unsigned char*,bool rev=false);
    ForwardingHeader(const ForwardingHeader&);
    ForwardingHeader& operator=(const ForwardingHeader&);
    ~ForwardingHeader();
    WritablePacket* apply(Packet *) const;
    //  private:
    int len;
    unsigned char* data;
  };

  typedef HashTable<IPFlowID,ForwardingHeader> Map;

  RevRoute();
  ~RevRoute();
  const char *class_name() const { return "RevRoute"; }
  const char *port_count() const { return "2/2"; }
  const char *processing() const { return PUSH; }
  void push(int port, Packet *p);

private:
  Map _udp_map;
  Map _tcp_map;
};

CLICK_ENDDECLS
#endif
