#ifndef CLICK_STRIPFORWHEADER_HH
#define CLICK_STRIPFORWHEADER_HH
#include <click/element.hh>
#include <click/ipaddress.hh>
CLICK_DECLS

/*
=c
StripForwHeader

=s Forwarding Network
strips outermost Forwarding Header

=processing
agnostic

=d 
Strips the outermost Forwarding Header.

Note that StripForwHeader assumes that the Forwarding Header starts at
the beginning of the packet, hence you need to use StripIPHeader and
Strip(8) before processing packets iwth StripForwHeader.
Also note that StripForwHeader will set the network and transport
header annotation, assuming that an IP header is following.

=a RevRoute, NextHop, ForwHeader, ForwHeaderPrint */



class StripForwHeader : public Element {
public:
  StripForwHeader();
  ~StripForwHeader();
  const char *class_name() const { return "StripForwHeader"; }
  const char *port_count() const { return PORTS_1_1; }
  const char *processing() const { return PUSH; }
  void push(int port, Packet *p);
};

CLICK_ENDDECLS
#endif
